@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2016, 2018-2019 Laboratoire de Recherche et
+// Copyright (C) 2012-2016, 2018-2020 Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -24,7 +24,7 @@
 #include "error.h"
 #include "argmatch.h"
 
-spot::postprocessor::output_type type = spot::postprocessor::TGBA;
+spot::postprocessor::output_type type = spot::postprocessor::GeneralizedBuchi;
 spot::postprocessor::output_pref pref = spot::postprocessor::Small;
 spot::postprocessor::output_pref comp = spot::postprocessor::Any;
 spot::postprocessor::output_pref sbacc = spot::postprocessor::Any;
@@ -50,9 +50,15 @@ static constexpr const argp_option options[] =
     { "generic", 'G', nullptr, 0,
       "any acceptance condition is allowed", 0 },
     { "tgba", OPT_TGBA, nullptr, 0,
-      "Transition-based Generalized Büchi Automaton (default)", 0 },
-    { "ba", 'B', nullptr, 0,
-      "Büchi Automaton (implies -S)", 0 },
+      "automaton with Generalized Büchi acceptance (default)", 0 },
+    { "gba", 0, nullptr, OPTION_ALIAS, nullptr, 0 },
+    { "sba", 'B', nullptr, 0,
+      "state-based Büchi Automaton (same as -S -b)", 0 },
+    // Historical name of the --sba option.
+    { "ba", 0, nullptr, OPTION_ALIAS, nullptr, 0 },
+    { "buchi", 'b', nullptr, 0,
+      "automaton with Büchi acceptance", 0 },
+    { "Buchi", 0, nullptr, OPTION_ALIAS, nullptr, 0 },
     { "monitor", 'M', nullptr, 0, "Monitor (accepts all finite prefixes "
       "of the given property)", 0 },
     { "complete", 'C', nullptr, 0, "output a complete automaton", 0 },
@@ -113,9 +119,15 @@ static const argp_option options_disabled[] =
     { "generic", 'G', nullptr, 0,
       "any acceptance is allowed (default)", 0 },
     { "tgba", OPT_TGBA, nullptr, 0,
-      "Transition-based Generalized Büchi Automaton", 0 },
-    { "ba", 'B', nullptr, 0,
-      "Büchi Automaton (with state-based acceptance)", 0 },
+      "automaton with Generalized Büchi acceptance", 0 },
+    { "gba", 0, nullptr, OPTION_ALIAS, nullptr, 0 },
+    { "sba", 'B', nullptr, 0,
+      "state-based Büchi Automaton (same as -S -b)", 0 },
+    // Historical name of the --sba option.
+    { "ba", 0, nullptr, OPTION_ALIAS, nullptr, 0 },
+    { "buchi", 'b', nullptr, 0,
+      "automaton with Büchi acceptance", 0 },
+    { "Buchi", 0, nullptr, OPTION_ALIAS, nullptr, 0 },
     { "monitor", 'M', nullptr, 0, "Monitor (accepts all finite prefixes "
       "of the given property)", 0 },
     { "complete", 'C', nullptr, 0, "output a complete automaton", 0 },
@@ -164,8 +176,12 @@ parse_opt_post(int key, char* arg, struct argp_state*)
       pref = spot::postprocessor::Any;
       pref_set = true;
       break;
+    case 'b':
+      type = spot::postprocessor::Buchi;
+      colored = spot::postprocessor::Any;
+      break;
     case 'B':
-      type = spot::postprocessor::BA;
+      type = spot::postprocessor::Buchi;
       colored = spot::postprocessor::Any;
       sbacc = spot::postprocessor::SBAcc;
       break;
@@ -250,7 +266,7 @@ parse_opt_post(int key, char* arg, struct argp_state*)
     case OPT_TGBA:
       if (automaton_format == Spin)
         error(2, 0, "--spin and --tgba are incompatible");
-      type = spot::postprocessor::TGBA;
+      type = spot::postprocessor::GeneralizedBuchi;
       colored = spot::postprocessor::Any;
       break;
     default:
