@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2014-2020 Laboratoire de Recherche et Développement
+// Copyright (C) 2014-2021 Laboratoire de Recherche et Développement
 // de l'Epita.
 //
 // This file is part of Spot, a model checking library.
@@ -140,125 +140,182 @@ namespace spot
     if (!is_existential())
       merge_univ_dests();
 
-    typedef graph_t::edge_storage_t tr_t;
-    g_.sort_edges_([](const tr_t& lhs, const tr_t& rhs)
-                   {
-                     if (lhs.src < rhs.src)
-                       return true;
-                     if (lhs.src > rhs.src)
-                       return false;
-                     if (lhs.dst < rhs.dst)
-                       return true;
-                     if (lhs.dst > rhs.dst)
-                       return false;
-                     return lhs.acc < rhs.acc;
-                     // Do not sort on conditions, we'll merge
-                     // them.
-                   });
-
     auto& trans = this->edge_vector();
     unsigned orig_size = trans.size();
     unsigned tend = orig_size;
-    unsigned out = 0;
-    unsigned in = 1;
-    // Skip any leading false edge.
-    while (in < tend && trans[in].cond == bddfalse)
-      ++in;
-    if (in < tend)
+
+    // When two transitions have the same (src,colors,dst),
+    // we can marge their conds.
+    auto merge_conds_and_remove_false = [&]()
+    {
+      typedef graph_t::edge_storage_t tr_t;
+      g_.sort_edges_([](const tr_t& lhs, const tr_t& rhs)
       {
-        ++out;
-        if (out != in)
-          trans[out] = trans[in];
-        for (++in; in < tend; ++in)
-          {
-            if (trans[in].cond == bddfalse) // Unusable edge
-              continue;
-            // Merge edges with the same source, destination, and
-            // acceptance.  (We test the source last, because this is the
-            // most likely test to be true as edges are ordered by
-            // sources and then destinations.)
-            if (trans[out].dst == trans[in].dst
-                && trans[out].acc == trans[in].acc
-                && trans[out].src == trans[in].src)
-              {
-                trans[out].cond |= trans[in].cond;
-              }
-            else
-              {
-                ++out;
-                if (in != out)
-                  trans[out] = trans[in];
-              }
-          }
-      }
-    if (++out != tend)
-      trans.resize(out);
+        if (lhs.src < rhs.src)
+          return true;
+        if (lhs.src > rhs.src)
+          return false;
+        if (lhs.dst < rhs.dst)
+          return true;
+        if (lhs.dst > rhs.dst)
+          return false;
+        return lhs.acc < rhs.acc;
+        // Do not sort on conditions, we'll merge
+        // them.
+      });
 
-    tend = out;
-    out = 1;
-    in = 2;
+      unsigned out = 0;
+      unsigned in = 1;
 
+      // Skip any leading false edge.
+      while (in < tend && trans[in].cond == bddfalse)
+        ++in;
+      if (in < tend)
+        {
+          ++out;
+          if (out != in)
+            trans[out] = trans[in];
+          for (++in; in < tend; ++in)
+            {
+              if (trans[in].cond == bddfalse) // Unusable edge
+                continue;
+              // Merge edges with the same source, destination, and
+              // colors.  (We test the source last, because this is the
+              // most likely test to be true as edges are ordered by
+              // sources and then destinations.)
+              if (trans[out].dst == trans[in].dst
+                  && trans[out].acc == trans[in].acc
+                  && trans[out].src == trans[in].src)
+                {
+                  trans[out].cond |= trans[in].cond;
+                }
+              else
+                {
+                  ++out;
+                  if (in != out)
+                    trans[out] = trans[in];
+                }
+            }
+        }
+      if (++out != tend)
+        {
+          tend = out;
+          trans.resize(tend);
+        }
+    };
+
+    // When two transitions have the same (src,cond,dst), we can marge
+    // their colors.  This only works for Fin-less acceptance.
+    //
     // FIXME: We could should also merge edges when using
     // fin_acceptance, but the rule for Fin sets are different than
     // those for Inf sets, (and we need to be careful if a set is used
     // both as Inf and Fin)
-    if ((in < tend) && !acc().uses_fin_acceptance())
+    auto merge_colors = [&]()
+    {
+      if (tend < 2 || acc().uses_fin_acceptance())
+        return;
+      typedef graph_t::edge_storage_t tr_t;
+      g_.sort_edges_([](const tr_t& lhs, const tr_t& rhs)
       {
-        typedef graph_t::edge_storage_t tr_t;
-        g_.sort_edges_([](const tr_t& lhs, const tr_t& rhs)
-                       {
-                         if (lhs.src < rhs.src)
-                           return true;
-                         if (lhs.src > rhs.src)
-                           return false;
-                         if (lhs.dst < rhs.dst)
-                           return true;
-                         if (lhs.dst > rhs.dst)
-                           return false;
-                         bdd_less_than_stable lt;
-                         return lt(lhs.cond, rhs.cond);
-                         // Do not sort on acceptance, we'll merge
-                         // them.
-                       });
+        if (lhs.src < rhs.src)
+          return true;
+        if (lhs.src > rhs.src)
+          return false;
+        if (lhs.dst < rhs.dst)
+          return true;
+        if (lhs.dst > rhs.dst)
+          return false;
+        bdd_less_than_stable lt;
+        return lt(lhs.cond, rhs.cond);
+        // Do not sort on acceptance, we'll merge them.
+      });
 
-        for (; in < tend; ++in)
-          {
-            // Merge edges with the same source, destination,
-            // and conditions.  (We test the source last, for the
-            // same reason as above.)
-            if (trans[out].dst == trans[in].dst
-                && trans[out].cond.id() == trans[in].cond.id()
-                && trans[out].src == trans[in].src)
-              {
-                trans[out].acc |= trans[in].acc;
-              }
-            else
-              {
-                ++out;
-                if (in != out)
-                  trans[out] = trans[in];
-              }
-          }
-        if (++out != tend)
-          trans.resize(out);
-      }
+      // Start on the second position and try to merge it into the
+      // first one
+      unsigned out = 1;
+      unsigned in = 2;
 
+      for (; in < tend; ++in)
+        {
+          // Merge edges with the same source, destination,
+          // and conditions.  (We test the source last, for the
+          // same reason as above.)
+          if (trans[out].dst == trans[in].dst
+              && trans[out].cond.id() == trans[in].cond.id()
+              && trans[out].src == trans[in].src)
+            {
+              trans[out].acc |= trans[in].acc;
+            }
+          else
+            {
+              ++out;
+              if (in != out)
+                trans[out] = trans[in];
+            }
+        }
+      if (++out != tend)
+        {
+          tend = out;
+          trans.resize(tend);
+        }
+    };
+
+    // These next two lines used to be done in the opposite order in
+    // 2.9.x and before.
+    //
+    // However merging colors first is more likely to
+    // make parallel non-deterministic transition disappear.
+    //
+    // Consider:
+    //           (1)--a-{1}--->(2)
+    //           (1)--a-{2}--->(2)
+    //           (1)--!a-{1}-->(2)
+    // If colors are merged first, the first two transitions become one,
+    // and the result is more deterministic:
+    //           (1)--a-{1,2}->(2)
+    //           (1)--!a-{1}-->(2)
+    // If conditions were merged first, we would get the following
+    // non-deterministic transitions instead:
+    //           (1)--1-{1}--->(2)
+    //           (1)--a-{2}--->(2)
+    merge_colors();
+    merge_conds_and_remove_false();
+
+    // Merging some edges may turn a non-deterministic automaton
+    // into a deterministic one.
+    if (prop_universal().is_false() && tend != orig_size)
+      prop_universal(trival::maybe());
     g_.chain_edges_();
-
-    // Did we actually reduce the number of edges?
-    if (trans.size() != orig_size)
-      // Merging some edges may turn a non-deterministic automaton
-      // into a deterministic one.
-      if (prop_universal().is_false())
-        prop_universal(trival::maybe());
   }
 
-  void twa_graph::merge_states()
+  unsigned twa_graph::merge_states()
   {
     if (!is_existential())
       throw std::runtime_error(
           "twa_graph::merge_states() does not work on alternating automata");
 
+    typedef graph_t::edge_storage_t tr_t;
+    g_.sort_edges_([](const tr_t& lhs, const tr_t& rhs)
+    {
+      if (lhs.src < rhs.src)
+        return true;
+      if (lhs.src > rhs.src)
+        return false;
+      if (lhs.acc < rhs.acc)
+        return true;
+      if (lhs.acc > rhs.acc)
+        return false;
+      if (bdd_less_than_stable lt; lt(lhs.cond, rhs.cond))
+        return true;
+      if (rhs.cond != lhs.cond)
+        return false;
+      // The destination must be sorted last
+      // for our self-loop optimization to work.
+      return lhs.dst < rhs.dst;
+    });
+    g_.chain_edges_();
+
     const unsigned nb_states = num_states();
     std::vector<unsigned> remap(nb_states, -1U);
     for (unsigned i = 0; i != nb_states; ++i)
@@ -270,9 +327,39 @@ namespace spot
             if (std::equal(out1.begin(), out1.end(), out2.begin(), out2.end(),
                            [](const edge_storage_t& a,
                               const edge_storage_t& b)
-                           { return a.dst == b.dst && a.data() == b.data(); }))
+                           { return ((a.dst == b.dst
+                                      || (a.dst == a.src && b.dst == b.src))
+                                     && a.data() == b.data()); }))
             {
-              remap[i] = j;
+              remap[i] = (remap[j] != -1U) ? remap[j] : j;
+
+              // Because of the special self-loop tests we use above,
+              // it's possible that i can be mapped to remap[j] even
+              // if j was last compatible states found.  Consider the
+              // following cases, taken from an actual test case:
+              // 18 is equal to 5, 35 is equal to 18, but 35 is not
+              // equal to 5.
+              //
+              // State: 5
+              // [0&1&2] 8 {3}
+              // [!0&1&2] 10 {1}
+              // [!0&!1&!2] 18 {1}
+              // [!0&!1&2] 19 {1}
+              // [!0&1&!2] 20 {1}
+              //
+              // State: 18
+              // [0&1&2] 8 {3}
+              // [!0&1&2] 10 {1}
+              // [!0&!1&!2] 18 {1} // self-loop
+              // [!0&!1&2] 19 {1}
+              // [!0&1&!2] 20 {1}
+              //
+              // State: 35
+              // [0&1&2] 8 {3}
+              // [!0&1&2] 10 {1}
+              // [!0&!1&!2] 35 {1} // self-loop
+              // [!0&!1&2] 19 {1}
+              // [!0&1&!2] 20 {1}
               break;
             }
           }
@@ -292,7 +379,158 @@ namespace spot
       else
         s = -1U;
 
-    defrag_states(std::move(remap), st);
+    unsigned merged = num_states() - st;
+    if (merged)
+      defrag_states(remap, st);
+    return merged;
+  }
+
+  unsigned twa_graph::merge_states_of(bool stable,
+                                      const std::vector<bool>* to_merge_ptr)
+  {
+    if (!is_existential())
+      throw std::runtime_error(
+          "twa_graph::merge_states() does not work on alternating automata");
+
+    typedef graph_t::edge_storage_t tr_t;
+    if (stable)
+      g_.sort_edges_of_<true>([](const tr_t& lhs, const tr_t& rhs)
+      {
+        if (lhs.acc < rhs.acc)
+          return true;
+        if (lhs.acc > rhs.acc)
+          return false;
+        if (bdd_less_than_stable lt; lt(lhs.cond, rhs.cond))
+          return true;
+        if (rhs.cond != lhs.cond)
+          return false;
+        // The destination must be sorted last
+        // for our self-loop optimization to work.
+        return lhs.dst < rhs.dst;
+      });
+    else
+      g_.sort_edges_of_<false>([](const tr_t& lhs, const tr_t& rhs)
+      {
+        if (lhs.acc < rhs.acc)
+          return true;
+        if (lhs.acc > rhs.acc)
+          return false;
+        if (bdd_less_than lt; lt(lhs.cond, rhs.cond))
+          return true;
+        if (rhs.cond != lhs.cond)
+          return false;
+        // The destination must be sorted last
+        // for our self-loop optimization to work.
+        return lhs.dst < rhs.dst;
+      });
+
+    // Associates a hash value to a vector of classes
+    std::unordered_map<size_t, std::vector<std::set<unsigned>>> equiv_class_;
+
+    auto hash_state_ = [&](unsigned s)->size_t
+      {
+        // Hash the edges
+        // bottle_neck?
+        size_t h = fnv<size_t>::init;
+        for (const edge_storage_t& e : out(s))
+          {
+            h ^= knuth32_hash(e.dst);
+            h ^= knuth32_hash(e.cond.id());
+            h ^= e.acc.hash();
+            h = wang32_hash(h);
+          }
+        return h;
+      };
+
+    const unsigned nb_states = num_states();
+
+    std::vector<unsigned> comp_classes_; // Classes to be merged
+    for (unsigned i = 0; i != nb_states; ++i)
+      {
+        if (to_merge_ptr && !(*to_merge_ptr)[nb_states])
+          continue;
+
+        size_t hi = hash_state_(i);
+        auto equal_to_i_ = [&, outi = out(i)](unsigned j)
+          {
+            auto outj = out(j);
+            return std::equal(outi.begin(), outi.end(),
+                              outj.begin(), outj.end(),
+                              [](const edge_storage_t& a,
+                                 const edge_storage_t& b)
+                              { return ((a.dst == b.dst
+                                         || (a.dst == a.src && b.dst == b.src))
+                                         && a.data() == b.data()); });
+          };
+        comp_classes_.clear();
+        // get all compatible classes
+        // Candidate classes share a hash
+        // A state is compatible to a class if it is compatble
+        // to any of its states
+        auto& cand_classes = equiv_class_[hi];
+        unsigned n_c_classes = cand_classes.size();
+        // Built it in "reverse order" the idx of
+        // classes to be merged
+        for (unsigned nc = n_c_classes - 1; nc < n_c_classes; --nc)
+          if (std::any_of(cand_classes[nc].begin(),
+                          cand_classes[nc].end(),
+                          [&](unsigned j)
+                          {return equal_to_i_(j); }))
+            comp_classes_.push_back(nc);
+        // Possible results:
+        // 1) comp_classes_ is empty -> i gets its own class
+        // 2) fuse together all comp_classes and add i
+        if (comp_classes_.empty())
+          cand_classes.emplace_back(std::set<unsigned>({i}));
+        else
+          {
+            // Lowest idx
+            auto& base_class = cand_classes[comp_classes_.back()];
+            comp_classes_.pop_back(); // Keep this one
+            for (unsigned compi : comp_classes_)
+              {
+                // fuse with base and delete
+                base_class.insert(cand_classes[compi].begin(),
+                                  cand_classes[compi].end());
+                std::swap(cand_classes[compi], cand_classes.back());
+                cand_classes.pop_back();
+              }
+            // finally add the current state that caused all the merging
+            base_class.emplace_hint(base_class.end(), i);
+          }
+      };
+
+    // Now we have equivalence classes
+    // and a state can only be in exactly one.
+    // (Otherwise the classes would have fused)
+    // For each equiv class we take the first state as representative
+    // and redirect all incoming edges to this one.
+    std::vector<unsigned> remap(nb_states, -1U);
+    for (const auto& [_, class_v] : equiv_class_)
+      for (const auto& aclass : class_v)
+        {
+          (void)_; // please some versions of GCC
+          unsigned rep = *aclass.begin();
+          for (auto it = ++aclass.begin(); it != aclass.end(); ++it)
+            remap[*it] = rep;
+        }
+
+    for (auto& e: edges())
+      if (remap[e.dst] != -1U)
+        e.dst = remap[e.dst];
+
+    if (remap[get_init_state_number()] != -1U)
+      set_init_state(remap[get_init_state_number()]);
+
+    unsigned st = 0;
+    for (auto& s: remap)
+      if (s == -1U)
+        s = st++;
+      else
+        s = -1U;
+
+    defrag_states(remap, st);
+    return remap.size() - st;
   }
 
   void twa_graph::purge_unreachable_states(shift_action* f, void* action_data)
@@ -347,7 +585,7 @@ namespace spot
     if (f)
       (*f)(todo, action_data);
 
-    defrag_states(std::move(todo), current);
+    defrag_states(todo, current);
   }
 
   void twa_graph::purge_dead_states()
@@ -544,13 +782,13 @@ namespace spot
     if (prop_complete().is_false())
       prop_complete(trival::maybe());
 
-    defrag_states(std::move(useful), current);
+    defrag_states(useful, current);
 
     if (purge_unreachable_needed)
       purge_unreachable_states();
   }
 
-  void twa_graph::defrag_states(std::vector<unsigned>&& newst,
+  void twa_graph::defrag_states(std::vector<unsigned>& newst,
                                 unsigned used_states)
   {
     if (!is_existential())
@@ -624,7 +862,7 @@ namespace spot
                   }
                 else
                   {
-                    // register this new destination group, add et two
+                    // register this new destination group, add it to
                     // newst, and use the index in newst to relabel
                     // the state so that graph::degrag_states() will
                     // eventually update it to the correct value.
@@ -702,14 +940,13 @@ namespace spot
           }
       }
     init_number_ = newst[init_number_];
-    g_.defrag_states(std::move(newst), used_states);
+    g_.defrag_states(newst, used_states);
   }
 
   void twa_graph::remove_unused_ap()
   {
     if (ap().empty())
       return;
-    std::set<bdd> conds;
     bdd all = ap_vars();
     for (auto& e: g_.edges())
       {
@@ -786,6 +1023,35 @@ namespace spot
     set_named_prop("state-names", names.release());
   }
 
+  void twa_graph::kill_state(unsigned state)
+  {
+    auto t = g_.out_iteraser(state);
+    while (t)
+      t.erase();
+    // A complete automaton is unlikely to stay
+    // complete after killing a state.
+    if (prop_complete().is_true())
+      prop_complete(trival::maybe());
+    prop_stutter_invariant(trival::maybe());
+    // Many properties are preserved by state removal, and may even
+    // become true if they were false before and the appropriate
+    // states are removed.
+    if (prop_universal().is_false())
+      prop_universal(trival::maybe());
+    if (prop_inherently_weak().is_false())
+      prop_inherently_weak(trival::maybe());
+    if (prop_weak().is_false())
+      prop_weak(trival::maybe());
+    if (prop_very_weak().is_false())
+      prop_very_weak(trival::maybe());
+    if (prop_terminal().is_false())
+      prop_terminal(trival::maybe());
+    if (prop_unambiguous().is_false())
+      prop_unambiguous(trival::maybe());
+    if (prop_semi_deterministic().is_false())
+      prop_semi_deterministic(trival::maybe());
+  }
+
   void twa_graph::dump_storage_as_dot(std::ostream& out,
                                       const char* opt) const
   {
@@ -896,9 +1162,14 @@ namespace spot
 
             if (!named_prop_.empty())
               {
-                out << "namedprops [label=\"named properties:\n";
+                // GraphiViz 2.46.0 has a bug where plain newlines in
+                // quoted strings are ignored.  See
+                // https://gitlab.com/graphviz/graphviz/-/issues/1931
+                // A workaround is to use emit \n instead of the
+                // actual new line.
+                out << "namedprops [label=\"named properties:\\n";
                 for (auto p: named_prop_)
-                  escape_html(out, p.first) << '\n';
+                  escape_html(out, p.first) << "\\n";
                 out << "\"]\n";
               }
           }
@@ -926,6 +1197,17 @@ namespace spot
     copy(const const_twa_ptr& aut, twa::prop_set p,
          bool preserve_names, unsigned max_states)
     {
+      // If the input is a twa_graph and the number of states is not
+      // restricted, simply use the make_twa_graph variant for
+      // twa_graph.  Not only is this faster, but this is also
+      // necessary as a workaround for Swig-3 calling the wrong copy
+      // of make_twa_graph because it tests if twa match before
+      // testing twa_graph (swig-4 seems fixed).
+      const_twa_graph_ptr aut_g =
+        std::dynamic_pointer_cast<const twa_graph>(aut);
+      if (max_states == -1U && aut_g)
+        return make_twa_graph(aut_g, p, preserve_names);
+
       twa_graph_ptr out = make_twa_graph(aut->get_dict());
       out->copy_acceptance_of(aut);
       out->copy_ap_of(aut);
@@ -938,7 +1220,6 @@ namespace spot
       typedef std::map<unsigned, unsigned> hmap;
       hmap* ohstates = nullptr;
       hmap* ohedges = nullptr;
-      const_twa_graph_ptr aut_g = nullptr;
       // New highlighting maps
       hmap* nhstates = nullptr;
       hmap* nhedges = nullptr;
@@ -950,7 +1231,6 @@ namespace spot
 
           // If the input is a twa_graph and we were asked to preserve
           // names, also preserve highlights.
-          aut_g = std::dynamic_pointer_cast<const twa_graph>(aut);
           if (aut_g)
             {
               ohstates = aut->get_named_prop<hmap>("highlight-states");
