@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2012, 2014-2020 Laboratoire de Recherche
+// Copyright (C) 2011, 2012, 2014-2021 Laboratoire de Recherche
 // et Développement de l'Epita (LRDE).
 // Copyright (C) 2003, 2004  Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
@@ -82,6 +82,7 @@ namespace spot
       std::string* graph_name_ = nullptr; // name for the digraph
       std::map<std::pair<int, int>, int> univ_done;
       std::vector<bool> true_states_;
+      std::vector<bool>* state_player_;
 
       acc_cond::mark_t inf_sets_ = {};
       acc_cond::mark_t fin_sets_ = {};
@@ -114,6 +115,8 @@ namespace spot
       const char* nl_ = "\\n";
       const char* label_pre_ = "label=\"";
       char label_post_ = '"';
+      bool opt_id_ = false;
+      std::string opt_id_graph_;
 
       const_twa_graph_ptr aut_;
       std::string opt_font_;
@@ -123,6 +126,8 @@ namespace spot
       unsigned max_states_ = -1U; // related to max_states_given_
 
       bool opt_shared_univ_dest_ = true;
+      bool opt_mealy_ = false;
+      bdd opt_mealy_output_;
 
     public:
 
@@ -278,6 +283,18 @@ namespace spot
             case 'h':
               opt_vertical_ = false;
               break;
+            case 'i':
+              if (*options == '(')
+                {
+                  auto* end = strchr(++options, ')');
+                  if (!end)
+                    throw std::runtime_error
+                      ("unclosed \"i(...\" for print_dot()");
+                  opt_id_graph_ = std::string(options, end - options);
+                options = end + 1;
+                }
+              opt_id_ = true;
+              break;
             case 'k':
               opt_state_labels_ = true;
               break;
@@ -441,9 +458,8 @@ namespace spot
       }
 
       std::ostream&
-      format_label(std::ostream& os, bdd label) const
+      format_label(std::ostream& os, formula f) const
       {
-        formula f = bdd_to_formula(label, aut_->get_dict());
         if (opt_latex_)
           {
             print_sclatex_psl(os << '$', f) << '$';
@@ -460,6 +476,12 @@ namespace spot
         return escape_for_output(os, s);
       }
 
+      std::ostream&
+      format_label(std::ostream& os, bdd label) const
+      {
+        return format_label(os, bdd_to_formula(label, aut_->get_dict()));
+      }
+
       std::ostream&
       format_state_label(std::ostream& os, unsigned s) const
       {
@@ -475,6 +497,67 @@ namespace spot
         return format_label(os, label);
       }
 
+      std::ostream&
+      format_mealy(std::ostream& os,
+                   bdd in,
+                   bdd out) const
+      {
+        format_label(os, in);
+        if (opt_html_labels_)
+          os << "</TD><TD>/</TD><TD ALIGN=\"LEFT\" BGCOLOR=\"#ffe5f1\">";
+        else
+          os << " / ";
+        format_label(os, out);
+        return os;
+      }
+
+      std::ostream&
+      format_mealy(std::ostream& os, bdd cond) const
+      {
+        if (opt_html_labels_)
+          os << ("<TABLE BORDER=\"0\"><TR><TD ALIGN=\"RIGHT\""
+                 " BGCOLOR=\"#e9f4fb\">");
+        // Is the label is separable?
+        bdd in = bdd_exist(cond, opt_mealy_output_);
+        bdd out = bdd_existcomp(cond, opt_mealy_output_);
+        if (cond == (in & out))
+          {
+            format_mealy(os, in, out);
+          }
+        else
+          {
+            // Otherwise, present it as a sum of separable labels.
+            std::map<bdd, bdd, bdd_less_than_stable> in_out;
+            bdd sup = bdd_support(cond);
+            for (bdd cond: minterms_of(cond, sup))
+              {
+                bdd in = bdd_exist(cond, opt_mealy_output_);
+                bdd out = bdd_existcomp(cond, opt_mealy_output_);
+                if (auto p = in_out.emplace(in, out); !p.second)
+                  p.first->second |= out;
+              }
+            std::map<bdd, bdd, bdd_less_than_stable> out_in;
+            for (auto [in, out]: in_out)
+              if (auto p = out_in.emplace(out, in); !p.second)
+                p.first->second |= in;
+            bool first = true;
+            for (auto [out, in]: out_in)
+              {
+                if (!first)
+                  {
+                    if (opt_html_labels_)
+                      os << ("</TD></TR><TR><TD ALIGN=\"RIGHT\""
+                             " BGCOLOR=\"#e9f4fb\">");
+                    else
+                      os << '\n';
+                  }
+                format_mealy(os, in, out);
+                first = false;
+              }
+          }
+        return os;
+      }
+
       std::string string_dst(int dst, int color_num = -1)
       {
         std::ostringstream tmp_dst;
@@ -515,8 +598,19 @@ namespace spot
                   os_ << 'T' << d << 'T' << dest;
                 else
                   os_ << d;
-                if (style && *style)
-                  os_ << " [" << style << ']';
+                if ((style && *style) || opt_id_)
+                  {
+                    os_ << " [";
+                    const char* sep = "";
+                    if (style && *style)
+                      {
+                        os_ << style;
+                        sep = ",";
+                      }
+                    if (opt_id_)
+                      os_ << sep << "id=\"E" << dest << 'E' << d << '\"';
+                    os_ << ']';
+                  }
                 os_ << '\n';
               }
             univ = 2;
@@ -627,6 +721,12 @@ namespace spot
               << "\"\n  node [fontname=\"" << opt_font_
               << "\"]\n  edge [fontname=\"" << opt_font_
               << "\"]\n";
+        if (opt_id_)
+          {
+            if (!opt_id_graph_.empty())
+              escape_str(os_ << "  id=\"", opt_id_graph_) << "\"\n";
+            os_ << "  node [id=\"S\\N\"]\n";
+          }
         // Always copy the environment variable into a static string,
         // so that we (1) look it up once, but (2) won't crash if the
         // environment is changed.
@@ -718,6 +818,8 @@ namespace spot
             if (dcircles_ && aut_->state_acc_sets(s))
               os_ << ", peripheries=2";
           }
+        if (state_player_ && state_player_->size() > s && (*state_player_)[s])
+          os_ << ", shape=\"diamond\"";
         if (highlight_states_)
           {
             auto iter = highlight_states_->find(s);
@@ -782,15 +884,28 @@ namespace spot
               }
             os_ << " [" << label_pre_;
             if (!opt_state_labels_ && opt_showlabel_)
-              format_label(os_, t.cond);
+              {
+                if (opt_mealy_)
+                  format_mealy(os_, t.cond);
+                else
+                  format_label(os_, t.cond);
+              }
             if (!mark_states_)
               if (auto a = t.acc)
                 {
                   if (!opt_state_labels_ && opt_showlabel_)
-                    os_ << nl_;
+                    {
+                      if (opt_mealy_ && opt_html_labels_)
+                        os_ << "</TD></TR><TR><TD COLSPAN=\"3\">";
+                      else
+                        os_ << nl_;
+                    }
                   output_mark(a);
                 }
-            os_ << label_post_;
+            if (opt_mealy_ && opt_html_labels_)
+              os_ << "</TD></TR></TABLE>>";
+            else
+              os_ << label_post_;
             if (opt_ordered_edges_ || opt_numbered_edges_)
               {
                 os_ << ", taillabel=\"";
@@ -802,8 +917,15 @@ namespace spot
                   os_ << '#' << aut_->get_graph().index_of_edge(t);
                 os_ << '"';
               }
+            if (opt_id_)
+              {
+                unsigned index = aut_->get_graph().index_of_edge(t);
+                os_ << ", id=\"E" << index << '"';
+                // Somehow \E must appear as \\E in tooltip in
+                // graphviz 2.43.0; this looks like a bug.
+                os_ << ", tooltip=\"\\\\E\\n#" << index << '"';
+              }
           }
-
         std::string highlight;
         int color_num = -1;
         if (highlight_edges_)
@@ -888,8 +1010,20 @@ namespace spot
                              return p.second >= palette_mod / 2;
                            }) != e;
           }
+        // setup Mealy machine rendering.
+        if (!aut->get_named_prop<std::vector<bool>>("state-player"))
+          if (auto p = aut->get_named_prop<bdd>("synthesis-outputs"))
+            {
+              if (aut->acc().is_t())
+                opt_show_acc_ = false;
+              bdd out = *p;
+              opt_mealy_output_ = out;
+              opt_mealy_ = true;
+            }
         incomplete_ =
           aut->get_named_prop<std::set<unsigned>>("incomplete-states");
+        state_player_ =
+          aut_->get_named_prop<std::vector<bool>>("state-player");
         graph_name_ = aut_->get_named_prop<std::string>("automaton-name");
         if (opt_name_)
           name_ = graph_name_;
@@ -961,6 +1095,8 @@ namespace spot
                   // May only occur if the call to
                   // determine_unknown_acceptance() above is removed.
                   os_ << "  color=orange\n";
+                if (opt_id_)
+                  os_ << "  id=\"SCC" << i << "\"\n";
 
                 if (name_ || opt_show_acc_)
                   {
@@ -1020,5 +1156,163 @@ namespace spot
     return os;
   }
 
+  namespace
+  {
+    class dotty_aig final
+    {
+      public:
+      bool vertical = true;
+      bool latex = false;
+      std::ostream& os_;
+
+      void
+      parse_opts(const char *options)
+      {
+        while (char c = *options++)
+          switch (c)
+          {
+          case 'h':
+          {
+            vertical = false;
+            break;
+          }
+          case 'v':
+          {
+            vertical = true;
+            break;
+          }
+          default:
+            throw std::runtime_error("unknown option for print_dot(): "s + c);
+          }
+      }
+
+      dotty_aig(std::ostream &os, const char *options)
+          : os_(os)
+      {
+        parse_opts(options ? options : "");
+      }
+
+      void print(const aig_ptr &circuit)
+      {
+        if (!circuit)
+          return;
+        const auto& in_names = circuit->input_names();
+        const auto& out_names = circuit->output_names();
+
+        auto num_latches = circuit->num_latches();
+        auto latches = circuit->next_latches();
+        auto num_inputs = circuit->num_inputs();
+        auto num_outputs = circuit->num_outputs();
+        auto outputs = circuit->outputs();
+        auto num_gates = circuit->num_gates();
+        auto gates = circuit->gates();
+
+        auto add_edge = [](std::ostream &stream, const unsigned src,
+                           const unsigned dst) {
+          auto src_gate = src & ~1;
+          stream << "  " << src_gate << " -> " << dst;
+          if (src & 1)
+            stream << " [arrowhead=dot]";
+          stream << '\n';
+        };
+
+        if (vertical)
+          os_ << "digraph \"\" {\n  rankdir = BT\n";
+        else
+          os_ << "digraph \"\" {\n  rankdir = LR\n";
+
+        os_ << "  {\n    rank = same\n"
+          "    node [shape=box,style=filled,fillcolor=\"#ffe6cc\"]\n";
+        for (unsigned i = 0; i < num_latches; ++i)
+          {
+            auto first_m_mod = circuit->latch_var(i) & ~1;
+            os_ << "    " << first_m_mod << " [label=\"L" << i << "_out\"]\n";
+          }
+        os_ << "  }\n";
+        if (num_gates)
+          {
+            os_ << "  node [shape=circle,style=solid]\n";
+            for (unsigned i = 0; i < num_gates; ++i)
+              if ((gates[i].first != 0) && (gates[i].second != 0))
+                os_ << "  " << circuit->gate_var(i) << '\n';
+          }
+        bool has_false = false;
+        if (num_latches)
+          {
+            os_ << "  {\n    rank=same\n"
+              "    node [shape=box,style=filled,"
+              "fillcolor=\"#ffe6cc\",label=\"\\N_in\"]\n";
+            for (unsigned i = 0; i < num_latches; ++i)
+              {
+                os_ << "    L" << i << '\n';
+                if (latches[i] <= 1)
+                  has_false = true;
+              }
+            os_ << "  }\n";
+          }
+        const char* out_pos = vertical ? ":s" : ":w";
+        const char* rotate = vertical ? "" : ",orientation=-90";
+        if (num_outputs)
+          {
+            os_ << "  {\n    rank = sink\n    "
+              "node [shape=invtriangle,style=filled,fillcolor=\"#ffe5f1\""
+                << rotate << "]\n";
+            for (unsigned i = 0; i < num_outputs; ++i)
+              {
+                os_ << "    o" << i << " [label=\"" << out_names[i] << "\"]\n";
+                if (outputs[i] <= 1)
+                  has_false = true;
+              }
+            os_ << "  }\n";
+          }
+        if (num_inputs | has_false)
+          {
+            os_ << "  {\n    rank=source\n"
+              "    node [shape=triangle,style=filled,fillcolor=\"#e9f4fb\""
+                << rotate << "]\n";
+            for (unsigned i = 0; i < num_inputs; ++i)
+              os_ << "    " << circuit->input_var(i)
+                  << " [label=\"" << in_names[i] << "\"]\n";
+            if (has_false)
+              os_ <<
+                "    0 [shape=box,fillcolor=\"#ffe6cc\",label=\"False\"]\n";
+            os_ << "  }\n";
+          }
+        for (unsigned i = 0; i < num_outputs; ++i)
+          {
+            auto z = outputs[i];
+            os_ << "  " << (z & ~1) << " -> o" << i << out_pos;
+            if (z & 1)
+              os_ << " [arrowhead=dot]";
+            os_ << '\n';
+          }
+        for (unsigned i = 0; i < num_latches; ++i)
+          {
+            os_ << "  " << (latches[i] & ~1) << " -> L" << i;
+            if (latches[i] & 1)
+              os_ << " [arrowhead=dot]";
+            os_ << '\n';
+          }
+        for (unsigned i = 0; i < num_gates; ++i)
+          if ((gates[i].first != 0) && (gates[i].second != 0))
+            {
+              auto gate_var = circuit->gate_var(i);
+              add_edge(os_, gates[i].first, gate_var);
+              add_edge(os_, gates[i].second, gate_var);
+            }
+        os_ << "}\n";
+      }
+    };
+  }
+
+  std::ostream &
+  print_dot(std::ostream &os, aig_ptr circuit, const char* options)
+  {
+    dotty_aig d(os, options);
+    if (!circuit)
+      return os;
+    d.print(circuit);
+    return os;
+  }
 
 }
