@@ -384,9 +384,8 @@ namespace spot
     hash_set* non_final;
 
     twa_graph_ptr det_a;
-
+    bool has_sinks = false;
     {
-      power_map pm;
       bool input_is_det = is_deterministic(a);
       if (input_is_det)
         {
@@ -394,7 +393,34 @@ namespace spot
         }
       else
         {
-          det_a = tgba_powerset(a, pm, aborter);
+          // Find any accepting sink state, to speed up the
+          // determinization by merging all states containing a sink
+          // state.
+          std::vector<unsigned> acc_sinks;
+          unsigned ns = a->num_states();
+          if (!a->prop_terminal().is_true())
+            {
+              // We only consider as "accepting sink" any state
+              // that has an accepting self-loop labeled by true.
+              for (unsigned n = 0; n < ns; ++n)
+                for (auto& e: a->out(n))
+                  if (e.dst == n && e.cond == bddtrue
+                      && a->acc().accepting(e.acc))
+                    {
+                      acc_sinks.push_back(n);
+                      break;
+                    }
+            }
+          else
+            {
+              // All accepting SCCs are terminal.
+              scc_info si(a, scc_info_options::NONE);
+              for (unsigned n = 0; n < ns; ++n)
+                if (si.is_accepting_scc(si.scc_of(n)))
+                  acc_sinks.push_back(n);
+            }
+          has_sinks = !acc_sinks.empty();
+          det_a = tgba_powerset(a, aborter, &acc_sinks);
           if (!det_a)
             return nullptr;
         }
@@ -424,6 +450,13 @@ namespace spot
           for (unsigned m = 0; m < scc_count; ++m)
             is_accepting_scc[m] = sm.is_accepting_scc(m);
         }
+      else if (a->prop_terminal())
+        {
+          // tgba_powerset has gathered all terminal states on
+          // powerstate 0.
+          if (has_sinks)
+            is_accepting_scc[sm.scc_of(0)] = true;
+        }
       else
         {
           twa_graph_ptr prod = spot::product(a, det_a, aborter);
