@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2020 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2021 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -48,7 +48,7 @@ namespace spot
     static twa_graph_ptr
     ensure_ba(twa_graph_ptr& a)
     {
-      if (a->num_sets() == 0)
+      if (a->acc().is_t())
         {
           auto m = a->set_buchi();
           for (auto& t: a->edges())
@@ -69,11 +69,14 @@ namespace spot
         degen_lowinit_ = opt->get("degen-lowinit", 0);
         degen_remscc_ = opt->get("degen-remscc", 1);
         det_scc_ = opt->get("det-scc", 1);
-        det_simul_ = opt->get("det-simul", 1);
+        det_simul_ = opt->get("det-simul", -1);
         det_stutter_ = opt->get("det-stutter", 1);
         det_max_states_ = opt->get("det-max-states", -1);
         det_max_edges_ = opt->get("det-max-edges", -1);
         simul_ = opt->get("simul", -1);
+        simul_method_ = opt->get("simul-method", -1);
+        dpa_simul_ = opt->get("dpa-simul", -1);
+        dba_simul_ = opt->get("dba-simul", -1);
         scc_filter_ = opt->get("scc-filter", -1);
         ba_simul_ = opt->get("ba-simul", -1);
         tba_determinisation_ = opt->get("tba-det", 0);
@@ -85,6 +88,9 @@ namespace spot
         state_based_ = opt->get("state-based", 0);
         wdba_minimize_ = opt->get("wdba-minimize", -1);
         gen_reduce_parity_ = opt->get("gen-reduce-parity", 1);
+        simul_max_ = opt->get("simul-max", 4096);
+        wdba_det_max_ = opt->get("wdba-det-max", 4096);
+        simul_trans_pruning_ = opt->get("simul-trans-pruning", 512);
 
         if (sat_acc_ && sat_minimize_ == 0)
           sat_minimize_ = 1;        // Dicho.
@@ -110,19 +116,43 @@ namespace spot
   twa_graph_ptr
   postprocessor::do_simul(const twa_graph_ptr& a, int opt) const
   {
+    if (opt == 0)
+      return a;
+    if (simul_max_ > 0 && static_cast<unsigned>(simul_max_) < a->num_states())
+      return a;
+
+    static unsigned sim = [&]()
+    {
+      if (simul_method_ != -1)
+        return simul_method_;
+
+      char* s = getenv("SPOT_SIMULATION_REDUCTION");
+      return s ? *s - '0' : 0;
+    }();
+
+    if (sim == 2)
+      opt += 3;
+
+    // FIXME: simulation-based reduction now have work-arounds for
+    // non-separated sets, so we can probably try them.
     if (!has_separate_sets(a))
       return a;
     switch (opt)
       {
-      case 0:
-        return a;
       case 1:
-        return simulation(a);
+        return simulation(a, simul_trans_pruning_);
       case 2:
-        return cosimulation(a);
+        return cosimulation(a, simul_trans_pruning_);
       case 3:
+        return iterated_simulations(a, simul_trans_pruning_);
+      case 4:
+        return reduce_direct_sim(a);
+      case 5:
+        return reduce_direct_cosim(a);
+      case 6:
+        return reduce_iterated(a);
       default:
-        return iterated_simulations(a);
+        return iterated_simulations(a, simul_trans_pruning_);
       }
   }
 
@@ -131,20 +161,31 @@ namespace spot
   {
     if (ba_simul_ <= 0)
       return a;
+    if (simul_max_ > 0 && static_cast<unsigned>(simul_max_) < a->num_states())
+      return a;
     switch (opt)
       {
       case 0:
         return a;
       case 1:
-        return simulation_sba(a);
+        return simulation_sba(a, simul_trans_pruning_);
       case 2:
-        return cosimulation_sba(a);
+        return cosimulation_sba(a, simul_trans_pruning_);
       case 3:
       default:
-        return iterated_simulations_sba(a);
+        return iterated_simulations_sba(a, simul_trans_pruning_);
       }
   }
 
+  twa_graph_ptr
+  postprocessor::choose_degen(const twa_graph_ptr& a) const
+  {
+    if (state_based_)
+      return do_degen(a);
+    else
+      return do_degen_tba(a);
+  }
+
   twa_graph_ptr
   postprocessor::do_degen(const twa_graph_ptr& a) const
   {
@@ -164,15 +205,6 @@ namespace spot
                             degen_lowinit_, degen_remscc_);
   }
 
-  static void
-  force_buchi(twa_graph_ptr& a)
-  {
-    assert(a->acc().is_t());
-    acc_cond::mark_t m = a->set_buchi();
-    for (auto& e: a->edges())
-      e.acc = m;
-  }
-
   twa_graph_ptr
   postprocessor::do_scc_filter(const twa_graph_ptr& a, bool arg) const
   {
@@ -199,15 +231,18 @@ namespace spot
   twa_graph_ptr
   postprocessor::finalize(twa_graph_ptr tmp) const
   {
+    if (PREF_ != Any && level_ != Low)
+      tmp->remove_unused_ap();
     if (COMP_)
       tmp = complete(tmp);
     bool want_parity = type_ & Parity;
     if (want_parity && tmp->acc().is_generalized_buchi())
-      tmp = SBACC_ ? do_degen(tmp) : do_degen_tba(tmp);
-    if (SBACC_)
+      tmp = choose_degen(tmp);
+    assert(!!SBACC_ == state_based_);
+    if (state_based_)
       tmp = sbacc(tmp);
-    if (type_ == BA && tmp->acc().is_t())
-      force_buchi(tmp);
+    if (type_ == Buchi)
+      tmp = ensure_ba(tmp);
     if (want_parity)
       {
         reduce_parity_here(tmp, COLORED_);
@@ -232,13 +267,28 @@ namespace spot
     if (simul_ < 0)
       simul_ = (level_ == Low) ? 1 : 3;
     if (ba_simul_ < 0)
-      ba_simul_ = (level_ == High) ? 3 : 0;
+      ba_simul_ = (level_ == High) ? simul_ : 0;
+    int detaut_simul_default = (level_ != Low && simul_ > 0) ? 1 : 0;
+    if (dpa_simul_ < 0)
+      dpa_simul_ = detaut_simul_default;
+    if (dba_simul_ < 0)
+      dba_simul_ = detaut_simul_default;
+    if (det_simul_ < 0)
+      det_simul_ = simul_ > 0;
     if (scc_filter_ < 0)
       scc_filter_ = 1;
-    if (type_ == BA || SBACC_)
+    if (type_ == BA)
+      {
+        pref_ |= SBAcc;
+        type_ = Buchi;
+      }
+    if (SBACC_)
       state_based_ = true;
+    else if (state_based_)
+      pref_ |= SBAcc;
 
-    bool via_gba = (type_ == BA) || (type_ == TGBA) || (type_ == Monitor);
+    bool via_gba =
+      (type_ == Buchi) || (type_ == GeneralizedBuchi) || (type_ == Monitor);
     bool want_parity = type_ & Parity;
     if (COLORED_ && !want_parity)
       throw std::runtime_error("postprocessor: the Colored setting only works "
@@ -308,8 +358,8 @@ namespace spot
 
     if (PREF_ == Any && level_ == Low
         && (type_ == Generic
-            || type_ == TGBA
-            || (type_ == BA && a->is_sba())
+            || type_ == GeneralizedBuchi
+            || (type_ == Buchi && a->acc().is_buchi())
             || (type_ == Monitor && a->num_sets() == 0)
             || (want_parity && a->acc().is_parity())
             || (type_ == CoBuchi && a->acc().is_co_buchi())))
@@ -346,15 +396,14 @@ namespace spot
                 if (m->num_states() <= a->num_states())
                   a = m;
               }
-            a->remove_unused_ap();
           }
         return finalize(a);
       }
 
     if (PREF_ == Any)
       {
-        if (type_ == BA)
-          a = do_degen(a);
+        if (type_ == Buchi)
+          a = choose_degen(a);
         else if (type_ == CoBuchi)
           a = to_nca(a);
         return finalize(a);
@@ -386,7 +435,14 @@ namespace spot
     if (wdba_minimize)
       {
         bool reject_bigger = (PREF_ == Small) && (level_ <= Medium);
-        dba = minimize_obligation(a, f, nullptr, reject_bigger, aborter);
+        output_aborter* ab = aborter;
+        output_aborter wdba_aborter(wdba_det_max_ > 0 ?
+                                    (static_cast<unsigned>(wdba_det_max_)
+                                     + a->num_states()) : -1U);
+        if (!ab && PREF_ != Deterministic)
+          ab = &wdba_aborter;
+        dba = minimize_obligation(a, f, nullptr, reject_bigger, ab);
+
         if (dba
             && dba->prop_inherently_weak().is_true()
             && dba->prop_universal().is_true())
@@ -394,7 +450,7 @@ namespace spot
             // The WDBA is a BA, so no degeneralization is required.
             // We just need to add an acceptance set if there is none.
             dba_is_minimal = dba_is_wdba = true;
-            if (type_ == BA)
+            if (type_ == Buchi)
               ensure_ba(dba);
           }
         else
@@ -408,9 +464,9 @@ namespace spot
     // at hard levels if we want a small output.
     if (!dba || (level_ == High && PREF_ == Small))
       {
-        if (((SBACC_ && a->prop_state_acc().is_true())
-             || (type_ == BA && a->is_sba()))
-            && !tba_determinisation_)
+        if ((state_based_ && a->prop_state_acc().is_true())
+            && !tba_determinisation_
+            && (type_ != Buchi || a->acc().is_buchi()))
           {
             sim = do_sba_simul(a, ba_simul_);
           }
@@ -419,11 +475,11 @@ namespace spot
             sim = do_simul(a, simul_);
             // Degeneralize the result of the simulation if needed.
             // No need to do that if tba_determinisation_ will be used.
-            if (type_ == BA && !tba_determinisation_)
-              sim = do_degen(sim);
+            if (type_ == Buchi && !tba_determinisation_)
+              sim = choose_degen(sim);
             else if (want_parity && !sim->acc().is_parity())
               sim = do_degen_tba(sim);
-            else if (SBACC_ && !tba_determinisation_)
+            else if (state_based_ && !tba_determinisation_)
               sim = sbacc(sim);
           }
       }
@@ -434,21 +490,16 @@ namespace spot
     if (!dba && is_deterministic(sim))
       {
         std::swap(sim, dba);
-        // We postponed degeneralization above i case we would need
+        // We postponed degeneralization above in case we would need
         // to perform TBA-determinisation, but now it is clear
         // that we won't perform it.  So do degeneralize.
         if (tba_determinisation_)
           {
-            if (type_ == BA)
-              {
-                dba = do_degen(dba);
-                assert(is_deterministic(dba));
-              }
-            else if (SBACC_)
-              {
-                dba = sbacc(dba);
-                assert(is_deterministic(dba));
-              }
+            if (type_ == Buchi)
+              dba = choose_degen(dba);
+            else if (state_based_)
+              dba = sbacc(dba);
+            assert(is_deterministic(dba));
           }
       }
 
@@ -485,7 +536,7 @@ namespace spot
             // There is no point in running the reverse simulation on
             // a deterministic automaton, since all prefixes are
             // unique.
-            dba = simulation(tmp);
+            dba = do_simul(tmp, dba_simul_);
           }
         if (dba && PREF_ == Deterministic)
           {
@@ -495,23 +546,27 @@ namespace spot
         else
           {
             // degeneralize sim, because we did not do it earlier
-            if (type_ == BA)
-              sim = do_degen(sim);
+            if (type_ == Buchi)
+              sim = choose_degen(sim);
           }
       }
 
     if ((PREF_ == Deterministic && (type_ == Generic || want_parity)) && !dba)
       {
-        dba = tgba_determinize(to_generalized_buchi(sim),
-                               false, det_scc_, det_simul_, det_stutter_,
-                               aborter);
+        bool det_simul = det_simul_;
+        auto tba = to_generalized_buchi(sim);
+        if (simul_max_ > 0
+            && static_cast<unsigned>(simul_max_) < tba->num_states())
+          det_simul = false;
+        dba = tgba_determinize(tba,
+                               false, det_scc_, det_simul, det_stutter_,
+                               aborter, simul_trans_pruning_);
         // Setting det-max-states or det-max-edges may cause tgba_determinize
         // to fail.
+
         if (dba)
           {
-            dba = simplify_acc(dba);
-            if (level_ != Low)
-              dba = simulation(dba);
+            dba = do_simul(simplify_acc(dba), dpa_simul_);
             sim = nullptr;
           }
       }
@@ -529,12 +584,12 @@ namespace spot
           throw std::runtime_error
             ("postproc() not yet updated to mix sat-minimize and Generic");
         unsigned target_acc;
-        if (type_ == BA)
+        if (type_ == Buchi)
           target_acc = 1;
         else if (sat_acc_ != -1)
           target_acc = sat_acc_;
         else
-          // Take the number of acceptance conditions from the input
+          // Take the number of acceptance sets from the input
           // automaton, not from dba, because dba often has been
           // degeneralized by tba_determinize_check().  Make sure it
           // is at least 1.
@@ -611,8 +666,8 @@ namespace spot
 
     // Degeneralize the dba resulting from tba-determinization or
     // sat-minimization (which is a TBA) if requested and needed.
-    if (dba && !dba_is_wdba && type_ == BA
-        && !(dba_is_minimal && state_based_ && dba->num_sets() == 1))
+    if (dba && !dba_is_wdba && type_ == Buchi && state_based_
+        && !(dba_is_minimal && dba->num_sets() == 1))
       dba = degeneralize(dba);
 
     if (dba && sim)
@@ -641,8 +696,6 @@ namespace spot
 
     sim = dba ? dba : sim;
 
-    sim->remove_unused_ap();
-
     if (type_ == CoBuchi)
       {
         unsigned ns = sim->num_states();
