@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2015-2018 Laboratoire de Recherche et Développement
+// Copyright (C) 2015-2018, 2021 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -55,13 +55,19 @@ namespace spot
     std::vector<acc_cond::mark_t> one_in(ns, acc_cond::mark_t({}));
     std::vector<bool> true_state(ns, false);
     acc_cond::mark_t true_state_acc = {};
-    unsigned true_state_last;
+    // Do not initialize true_state_last, but prevent GCC from warning
+    // about a possible uninitialized use later.
+    unsigned true_state_last = unsigned();
     for (auto& e: old->edges())
       for (unsigned d: old->univ_dests(e.dst))
         if (si.scc_of(e.src) == si.scc_of(d))
           {
             common_in[d] &= e.acc;
             common_out[e.src] &= e.acc;
+            // Any state with an accepting edge labeled by true is a
+            // "true state".  We will merge all true states, and
+            // ignore other outgoing edges.  See issue #276 for an
+            // example.
             if (e.src == e.dst && e.cond == bddtrue
                 && old->acc().accepting(e.acc))
               {
@@ -87,14 +93,20 @@ namespace spot
     std::map<pair_t, unsigned> s2n;
 
     std::vector<std::pair<pair_t, unsigned>> todo;
+    auto* origin = new std::vector<unsigned>;
+    origin->reserve(ns);
+    res->set_named_prop("original-states", origin);
 
     auto new_state =
       [&](unsigned state, acc_cond::mark_t m) -> unsigned
       {
         bool ts = true_state[state];
-        if (ts)
+        if (ts) // Merge all true states.
           {
-            state = true_state_last; // Merge all true states.
+            // g++ 10.2 is afraid true_state_last might be
+            // uninitialized, however if ts==true, it means some true
+            // state were found, and the last true state is known.
+            state = true_state_last;
             m = {};
           }
         pair_t x(state, m);
@@ -102,6 +114,8 @@ namespace spot
         if (p.second)                // This is a new state
           {
             unsigned s = res->new_state();
+            assert(origin->size() == s);
+            origin->push_back(state);
             p.first->second = s;
             if (ts)
               {
@@ -170,6 +184,12 @@ namespace spot
       }
     res->merge_edges();
 
+    // Compose original-states with the any previously existing one.
+    if (auto old_orig_states =
+        old->get_named_prop<std::vector<unsigned>>("original-states"))
+      for (auto& s: *origin)
+        s = (*old_orig_states)[s];
+
     // If the automaton was marked as not complete or not universal,
     // and we have ignored some unreachable state, then it is possible
     // that the result becomes complete or universal.
