@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2013-2018, 2020 Laboratoire de Recherche et Développement
-// de l'Epita (LRDE).
+// Copyright (C) 2013-2018, 2020-2021 Laboratoire de Recherche et
+// Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -39,6 +39,7 @@ namespace spot
     relabel_bool_ = 4;
     tls_impl_ = -1;
     ltl_split_ = true;
+    exprop_ = -1;
 
     opt_ = opt;
     if (!opt)
@@ -60,11 +61,15 @@ namespace spot
         gf_guarantee_set_ = true;
       }
     ltl_split_ = opt->get("ltl-split", 1);
+    int tls_max_states = opt->get("tls-max-states", 64);
+    tls_max_states_ = std::max(0, tls_max_states);
+    exprop_ = opt->get("exprop", -1);
   }
 
   void translator::build_simplifier(const bdd_dict_ptr& dict)
   {
     tl_simplifier_options options(false, false, false);
+    options.containment_max_states = tls_max_states_;
     switch (level_)
       {
       case High:
@@ -139,7 +144,7 @@ namespace spot
             r2 = r2[0];
             ++leading_x;
           }
-        if (type_ == Generic || type_ == TGBA)
+        if (type_ == Generic || type_ == GeneralizedBuchi)
           {
             // F(q|u|f) = q|F(u)|F(f)  only for generic acceptance
             // G(q&e&f) = q&G(e)&G(f)
@@ -171,7 +176,7 @@ namespace spot
             // with disjunction, but it seems to generate larger automata
             // in many cases and it needs to be further investigated.  Maybe
             // this could be relaxed in the case of deterministic output.
-            (!r2.is(op::And) && (type_ == TGBA || type_ == BA)))
+            (!r2.is(op::And) && (type_ == GeneralizedBuchi || type_ == Buchi)))
           goto nosplit;
 
         op topop = r2.kind();
@@ -179,7 +184,7 @@ namespace spot
         std::vector<formula> oblg;
         std::vector<formula> susp;
         std::vector<formula> rest;
-        bool want_g = type_ == TGBA || type_ == BA;
+        bool want_g = type_ == GeneralizedBuchi || type_ == Buchi;
         for (formula child: r2)
           {
             if (child.is_syntactic_obligation())
@@ -210,7 +215,8 @@ namespace spot
           {
             // The only cases where we accept susp and rest to be both
             // non-empty is when doing Generic acceptance or TGBA.
-            if (!rest.empty() && !(type_ == Generic || type_ == TGBA))
+            if (!rest.empty()
+                && !(type_ == Generic || type_ == GeneralizedBuchi))
               {
                 rest.insert(rest.end(), susp.begin(), susp.end());
                 susp.clear();
@@ -222,7 +228,7 @@ namespace spot
           }
         // For TGBA and BA, we only split if there is something to
         // suspend.
-        if (susp.empty() && (type_ == TGBA || type_ == BA))
+        if (susp.empty() && (type_ == GeneralizedBuchi || type_ == Buchi))
           goto nosplit;
 
         option_map om_wos;
@@ -372,25 +378,26 @@ namespace spot
             if (gf_guarantee_ && PREF_ != Any)
               {
                 bool det = unambiguous || (PREF_ == Deterministic);
-                bool sba = type_ == BA || (pref_ & SBAcc);
-                if ((type_ & (BA | Parity | Generic)) || type_ == TGBA)
+                if ((type_ & (Buchi | Parity))
+                     || type_ == Generic
+                     || type_ == GeneralizedBuchi)
                   aut2 = gf_guarantee_to_ba_maybe(r, simpl_->get_dict(),
-                                                  det, sba);
-                if (aut2 && ((type_ == BA) || (type_ & Parity))
-                    && (pref_ & Deterministic))
+                                                  det, state_based_);
+                if (aut2 && (pref_ & Deterministic))
                   return finalize(aut2);
                 if (!aut2 && (type_ == Generic
                               || type_ & (Parity | CoBuchi)))
                   {
-                    aut2 = fg_safety_to_dca_maybe(r, simpl_->get_dict(), sba);
-                    if (aut2
-                        && (type_ & (CoBuchi | Parity))
-                        && (pref_ & Deterministic))
+                    aut2 = fg_safety_to_dca_maybe(r, simpl_->get_dict(),
+                                                  state_based_);
+                    if (aut2 && (pref_ & Deterministic))
                       return finalize(aut2);
                   }
               }
           }
-        bool exprop = unambiguous || level_ == postprocessor::High;
+        bool exprop = unambiguous
+          || (level_ == postprocessor::High && exprop_ != 0)
+          || exprop_ > 0;
         aut = ltl_to_tgba_fm(r, simpl_->get_dict(), exprop,
                              true, false, false, nullptr, nullptr,
                              unambiguous);
@@ -415,10 +422,20 @@ namespace spot
 
   twa_graph_ptr translator::run(formula* f)
   {
+    if (type_ == BA)
+      {
+        pref_ |= SBAcc;
+        type_ = Buchi;
+      }
+    if (pref_ & SBAcc)
+      state_based_ = true;
+    else if (state_based_)
+      pref_ |= SBAcc;
+
     if (simpl_owned_)
       {
         // Modify the options according to set_pref() and set_type().
-        // We do it for all translation, but really only the first one
+        // We do it for all translations, but really only the first one
         // matters.
         auto& opt = simpl_owned_->options();
         if (comp_susp_ > 0 || (ltl_split_ && type_ == Generic))
@@ -436,33 +453,61 @@ namespace spot
     // sub-formulas that are Boolean but do not have common terms.
     //
     // This rewriting is enabled only if the formula
-    //  1) has some Boolean subformula
-    //  2) has more than relabel_bool_ atomic propositions (the default
+    //  1) has more than relabel_bool_ atomic propositions (the default
     //     is 4, but this can be changed)
+    //  2) has some Boolean subformula
     //  3) relabel_bse() actually reduces the number of atomic
     //     propositions.
     relabeling_map m;
     formula to_work_on = *f;
     if (relabel_bool_ > 0)
       {
-        bool has_boolean_sub = false; // that is not atomic
         std::set<formula> aps;
-        to_work_on.traverse([&](const formula& f)
-                            {
-                              if (f.is(op::ap))
-                                aps.insert(f);
-                              else if (f.is_boolean())
-                                has_boolean_sub = true;
-                              return false;
-                            });
+        atomic_prop_collect(to_work_on, &aps);
         unsigned atomic_props = aps.size();
-        if (has_boolean_sub && (atomic_props >= (unsigned) relabel_bool_))
+
+        if (atomic_props >= (unsigned) relabel_bool_)
           {
-            formula relabeled = relabel_bse(to_work_on, Pnn, &m);
-            if (m.size() < atomic_props)
-              to_work_on = relabeled;
-            else
-              m.clear();
+            // Make a very quick simplification path before for
+            // Boolean subformulas, only only syntactic rules.  This
+            // is to help getting formulas of the form
+            //   FGp1 & FGp2 & ... & FGp32
+            // into the shape
+            //   FG(p1 & p2 & ... & p32)
+            // where relabeling will be much more efficient.
+            //
+            // We reuse all options of simpl_ (to preserve things
+            // like keep_top_xor or favor_event_univ).
+            tl_simplifier_options options = simpl_->options();
+            options.synt_impl = false;
+            options.containment_checks = false;
+            options.containment_checks_stronger = false;
+            options.nenoform_stop_on_boolean = true;
+            options.boolean_to_isop = false;
+            tl_simplifier simpl(options, simpl_->get_dict());
+            to_work_on = simpl.simplify(to_work_on);
+
+            // Do we have Boolean subformulas that are not atomic
+            // propositions?
+            bool has_boolean_sub = false;
+            to_work_on.traverse([&](const formula& f)
+                                {
+                                  if (f.is_boolean())
+                                    {
+                                      has_boolean_sub = true;
+                                      return true;
+                                    }
+                                  return false;
+                                });
+
+            if (has_boolean_sub)
+              {
+                formula relabeled = relabel_bse(to_work_on, Pnn, &m);
+                if (m.size() < atomic_props)
+                  to_work_on = relabeled;
+                else
+                  m.clear();
+              }
           }
       }
 
