@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 # -*- mode: python; coding: utf-8 -*-
-# Copyright (C) 2016, 2018 Laboratoire de Recherche et Développement de
+# Copyright (C) 2016, 2018, 2021 Laboratoire de Recherche et Développement de
 # l'EPITA.
 #
 # This file is part of Spot, a model checking library.
@@ -82,3 +82,44 @@ assert v == ()
 # FIXME: We should have a way to disable the following output, as it is not
 # part of HOA v1.
 assert acc.name() == "generalized-Streett 1 2"
+
+
+# issue #469.  This test is meaningful only if Spot is compiled with
+# --enable-max-accsets=64 or more.
+try:
+    m = spot.mark_t([33])
+    assert m.lowest() == m
+    n = spot.mark_t([33,34])
+    assert n.lowest() == m
+except RuntimeError as e:
+    if "Too many acceptance sets used." in str(e):
+        pass
+    else:
+        raise e
+
+
+# issue #468
+from gc import collect
+acc = spot.translate('a').acc()
+collect()
+assert acc == spot.acc_cond('Inf(0)')
+acc = spot.translate('b').get_acceptance()
+collect()
+assert acc == spot.acc_code('Inf(0)')
+
+
+c = spot.acc_cond('Fin(0)&Fin(1)&(Inf(2)|Fin(3))')
+m1 = c.fin_unit()
+m2 = c.inf_unit()
+assert m1 == [0,1]
+assert m2 == []
+c = spot.acc_cond('Inf(0)&Inf(1)&(Inf(2)|Fin(3))')
+m1 = c.fin_unit()
+m2 = c.inf_unit()
+assert m1 == []
+assert m2 == [0,1]
+c = spot.acc_cond('Inf(0)&Inf(1)|(Inf(2)|Fin(3))')
+m1 = c.fin_unit()
+m2 = c.inf_unit()
+assert m1 == []
+assert m2 == []
